# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .metadata import Metadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class EvaluationSet(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Name of the evaluation set
    """

    description: str = pydantic.Field()
    """
    Description of the evaluation set
    """

    scheduler: typing.Optional[str] = pydantic.Field(default=None)
    """
    Scheduler of the evaluation set
    """

    metadata: typing.Optional[Metadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
