# This file was auto-generated by Fern from our API Definition.

import typing

TraceTask = typing.Union[
    typing.Literal["retrieval", "generation", "feedback", "router", "system"],
    typing.Any,
]
