# This file was auto-generated by Fern from our API Definition.

import typing

MetadataBrowser = typing.Union[
    typing.Literal["chrome", "firefox", "safari", "edge", "opera", "unknown"],
    typing.Any,
]
